<?php
/*
Plugin Name: WP auto tag
Plugin URI: http://wwpteach.com/wp-auto-tag
Description: Automatically add relevant tags to new posts. 自動化的幫文章添加標籤。
Version: 0.2.2
Author: Richer Yang
Author URI: http://fantasyworld.idv.tw/
*/

register_activation_hook(__FILE__, 'WAT_init');
function WAT_init() {
	$test = get_option('WAT_options', array());
	if( count($test) == 0 ) {
		$options = array(
			'share_history' => array(),
			'last_get_ditc_time' => 0,
			'autotag_link' => 1,
			'autotag_link_maxtimes' => 1,
			'autotag_link_nofollow' => 1,
			'autotag_link_addin' => array(
				'home' => false,
				'single' => true,
				'archive' => false,
				'search' => false
			),
			'related_post' => 1,
			'related_post_aside_tag' => 1,
			'related_post_title' => 'Related Posts',
			'related_post_count' => 4,
			'related_post_img' => 0,
			'related_old_count' => 0,
			'analyitc_ngram_type' => 'r',
			'analyitc_max_words' => '6',
			'analyitc_max_tags' => '10'
		);
		add_option('WAT_options', $options, 'WP auto tag Options');
		add_option('WAT_version', '0.2.2', 'WP auto tag version');
	}
}

define('WAT_API_HOST', 'http://tagditc.info/');
$WAT_item = new WP_auto_tag();
include($WAT_item->base_dir . '/include/cronjob.php');
include($WAT_item->base_dir . '/include/function.php');

class WP_auto_tag {
	private $options;
	private $no_replace;
	private $analytics;
	public $base_dir;

	public function __construct() {
		$this->options = get_option('WAT_options');
		$this->base_dir = dirname(__FILE__);

		if( is_admin() ) {
			load_plugin_textdomain('WAT', false, dirname(plugin_basename(__FILE__)) . '/languages/');
			include($this->base_dir . '/include/update.php');
			include($this->base_dir . '/include/options.php');

			add_action('add_meta_boxes_post', array(&$this, 'add_meta_box'));
			add_action('wp_ajax_wp-auto-tag', array(&$this, 'ajax_make_tags'));
			add_filter('plugin_action_links', array(&$this, 'add_setting_links'), 10, 2);
			if( $this->options['related_post'] ) {
				include($this->base_dir . '/include/widget.php');
				add_action('publish_post', array(&$this, 'related_post'), 9999, 2);
			}
		} else {
			if( $this->options['autotag_link'] ) {
				add_filter('the_content', array(&$this, 'auto_taglink'), 50);
			}
			if( $this->options['related_post'] ) {
				add_action('init', array(&$this, 'enqueue_scripts'));
				add_action('init', array(&$this, 'enqueue_style'));
				include($this->base_dir . '/include/widget.php');
				if( $this->options['related_post_add_content'] ) {
					add_filter('the_content', array(&$this, 'related_post_add'), 51);
				}
			}
		}
	}

	public function add_meta_box() {
		add_action('admin_head', array(&$this, 'add_meta_box_scripts'));
		add_meta_box('tagsdiv-wpautotag', __('WP Auto Tag', 'WAT'), array(&$this, 'show_meta_box'), 'post', 'side', 'default');
	}

	public function show_meta_box($post) {
		if( $this->options['last_get_ditc_time'] > 0 ) {
			$date_format = get_option('date_format');
			$get_info = date($date_format, $this->options['last_get_ditc_time']) . __(' get the Dict file.', 'WAT');
		} else {
			$get_info = '<a href="admin.php?page=WAT_share">' . __('Go to get the Dict file.', 'WAT') . '</a>';
		}
		echo('<div class="inside"><p><input type="button" class="button doautotag" value="' . __('Analysis tag', 'WAT') . '"><br />' . $get_info . '</p></div>');
	}

	public function add_meta_box_scripts() {
		echo <<<END
<script type="text/javascript">
jQuery(document).ready(function($) {
	$('.doautotag').click(function(){
		var _item = $('.doautotag');
		var _content, _value;
		_content = tinymce.DOM.get('content').value;
		_value = _item.attr('value');
		_item.attr('disabled', true).attr('value', 'analyzing! Please wait');
		jQuery.post(ajaxurl, {
			action: 'wp-auto-tag',
			content: _content,
			title: $('#title').attr('value')
		}, function(data){
			$('#new-tag-post_tag').attr('value', data);
			_item.attr('disabled', false).attr('value', _value);
		});
	});
});
</script>
END;
	}

	public function ajax_make_tags() {
		$title = isset($_POST['title']) ? trim($_POST['title']) : '';
		$content = isset($_POST['content']) ? trim($_POST['content']) : '';
		$autotags = $this->make_tags($title, $content);
		echo(implode(',', $autotags));
		exit();
	}

	public function add_setting_links($links, $file) {
		if( $file == plugin_basename(__FILE__) ) {
			$links[] = '<a href="admin.php?page=WAT_options">' . __('Options', 'WAT') . '</a>';
		}
		return $links;
	}

	public function related_post($post_id, $post_data) {
		if( function_exists('curl_init') ) {
			$file = dirname(__FILE__) . '/related_post.data.gz';
			$related_post = array(
				'web_name' => get_bloginfo('blogname'),
				'locale' => get_locale(),
				'owner_url' => get_bloginfo('wpurl'),
				'post_id' => $post_id,
				'post_title' => $post_data->post_title,
				'post_url' => get_permalink($post_id),
				'post_thumbnail_url' => '',
				'post_tags' => array(),
				'post_auto_tags' => array()
			);
			$post_auto_tags = $this->make_tags($post_data->post_title, $post_data->post_content);
			$clear_content = $this->analytics->get_SimpleString();
			$tags = get_the_tags($post_id);
			if( is_array($tags) && count($tags) > 0 ) {
				foreach( $tags AS $tag ) {
					$related_post['post_tags'][] = sprintf('%s,%s,%s,%d', $tag->name, $tag->count, get_tag_link($tag), mb_substr_count($clear_content, $tag->name));
				}
			}
			if( count($post_auto_tags) > 0 ) {
				foreach( $post_auto_tags AS $tag_name ) {
					$tag = get_term_by('name', $tag_name, 'post_tag');
					$count = isset($tag->count) ? $tag->count : 0;
					$related_post['post_auto_tags'][] = sprintf('%s,%s,%s,%d', $tag_name, $count, get_tag_link($tag), mb_substr_count($clear_content, $tag_name));
				}
			}
			if( $this->options['related_post'] ) {
				$post_thumbnail_id = get_post_thumbnail_id($post_id);
				if( $post_thumbnail_id ) {
					$image = wp_get_attachment_image_src($post_thumbnail_id, 'post-thumbnail', false);
					if( $image ) {
						list($src, $width, $height) = $image;
						$related_post['post_thumbnail_url'] = $src;
					}
				}
			}
			$fp = gzopen($file, 'w');
			gzwrite($fp, serialize($related_post));
			gzclose($fp);
			$fields = array(
				'action' => 'relatedpost',
				'publish_shore' => $this->options['related_post'],
				'postdata' => '@' . $file,
				'analytic_version' => '20121023'
			);
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, WAT_API_HOST . 'get_api.php');
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
			curl_setopt($ch, CURLOPT_COOKIE, '');
			curl_setopt($ch, CURLOPT_TIMEOUT, 15);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
			$body = @curl_exec($ch);
			$info = @curl_getinfo($ch);
			@curl_close($ch);
			unlink($file);
		}
	}

	public function auto_taglink($content) {
		$out = true;
		if( is_home() && $this->options['autotag_link_addin']['home'] ) {
			$out = false;
		}
		if( is_single() && $this->options['autotag_link_addin']['single'] ) {
			$out = false;
		}
		if( is_archive() && $this->options['autotag_link_addin']['archive'] ) {
			$out = false;
		}
		if( is_search() && $this->options['autotag_link_addin']['search'] ) {
			$out = false;
		}
		if( $out ) {
			return $content;
		}

		global $post;
		$this->no_replace = '&&&';
		while( strpos($content, $this->no_replace) !== false ) {
			$this->no_replace .= '&';
		}
		$search = array();
		$replace = array();
		$tags = get_the_terms($post->ID, 'post_tag');
		if( $this->options['autotag_link_nofollow'] ) {
			$pre_replace = '<a class="autotag-link" href="%s" rel="tag nofollow">%s</a>';
		} else {
			$pre_replace = '<a class="autotag-link" href="%s" rel="tag">%s</a>';
		}
		if( is_array($tags) && count($tags) > 0 ) {
			foreach( $tags as $tag ) {
				$replace = sprintf($pre_replace, esc_url(get_term_link($tag, 'post_tag')), $tag->name);
				$content = preg_replace('@(<a[^>]*>[^<]*)' . preg_quote($tag->name) . '([^<]*<\/a[^>]*>)@si', '$1' . $this->no_replace . '$2', $content);
				$content = preg_replace_callback('@(<\/?[^>]*)(' . preg_quote($tag->name) . ')([^>]*>)@si', array(&$this, 'replace_taglink'), $content);
				$content = $this->mb_str_replace($tag->name, $replace, $content, $this->options['autotag_link_maxtimes']);
				$content = $this->mb_str_replace($this->no_replace, $tag->name, $content);
			}
		}
		return $content;
	}

	private function replace_taglink($matches) {
		if( strpos($matches[1], $matches[2]) !== false ) {
			$matches[1] = preg_replace_callback('@(<\/?[^>]*)(' . $matches[2] . ')([^>]*>)@si', array(&$this, 'replace_taglink'), $matches[1] . $matches[3]);
			$matches[1] = substr($matches[1], 0, 0 - strlen($matches[3]));
		}
		return $matches[1] . $this->no_replace . $matches[3];
	}

	public function enqueue_scripts() {
		wp_register_script('wat-js', plugins_url('wp_auto_tag.js', __FILE__), array('jquery'), '', true);
		wp_enqueue_script('wat-js');
	}

	public function enqueue_style() {
		wp_register_style('wat-css', plugins_url('wp_auto_tag.css', __FILE__));
		wp_enqueue_style('wat-css');
	}

	public function related_post_add($content) {
		if( is_singular() ) {
			$class = 'wat_hide';
			if( $this->options['related_post_img'] ) {
				$class .= ' has_thum';
			}
			$add_info = '<p>By <a href="http://tagditc.info/" >TagDitc</a>';
			$add_info .= '<input type="hidden" id="wat_count" value="' . intval($this->options['related_post_count']) . '">';
			$add_info .= '<input type="hidden" id="wat_host" value="' . sprintf('%u', crc32(strtolower(substr(get_bloginfo('wpurl'), 7)))) . strlen(get_bloginfo('wpurl')) . '">';
			$add_info .= '<input type="hidden" id="wat_post" value="' . get_the_ID() . '">';
			$add_info .= '<input type="hidden" id="wat_img" value="' . $this->options['related_post_img'] . '"></p>';
			if( $this->options['related_post_aside_tag'] ) {
				$content .= '<aside id="wat_related_post" class="' . $class . '"><h3>' . $this->options['related_post_title'] . '</h3><ul></ul>' . $add_info . '</aside>';
			} else {
				$content .= '<div id="wat_related_post" class="' . $class . '"><h3>' . $this->options['related_post_title'] . '</h3><ul></ul>' . $add_info . '</div>';
			}
		}
		return $content;
	}

	public function make_tags($title, $content) {
		include_once($this->base_dir . '/include/analytics_class.php');
		$content = $title . ' ' . $title . ' ' . $content;
		$this->analytics = new string_analytics();
		$this->analytics->SetOption($this->options['analyitc_ngram_type'], $this->options['analyitc_max_words'], $this->options['analyitc_max_tags']);
		$tags = get_terms('post_tag', array('hide_empty' => false));
		$max_tags = 0;
		$ditc = array();
		foreach( $tags as $key => $tag ) {
			$ditc[$tag->name] = $tag->count;
			$max_tags = max($max_tags, $tag->count);
		}
		$this->analytics->SetDict($ditc);
		if( $this->options['last_get_ditc_time'] > 0 ) {
			$ditc_file = $this->base_dir . '/include/share.ditc';
			if( !is_file($ditc_file) ) {
				to_get_ditc($ditc_file);
			} elseif( current_time('timestamp') - $this->options['last_get_ditc_time'] > 604800 ) {
				unlink($ditc_file);
				to_get_ditc($ditc_file);
			}
			if( is_file($ditc_file) ) {
				$ditc = unserialize(file_get_contents($ditc_file));
				if( is_array($ditc) ) {
					$ditc_max = reset($ditc);
					if( $ditc_max <= 0 ) {
						$ditc_max = $max_tags;
					}
					$diff = $max_tags / $ditc_max;
					if( $diff > 1 ) {
						$diff = 1;
					}
					$this->analytics->SetDict($ditc, $diff);
				}
			}
		}
		$this->analytics->SetSource($content);
		$this->analytics->StartAnalysis();
		$autotags = $this->analytics->get_finally();
		return $autotags;
	}

	private function mb_str_replace($search, $replace, $subject, $count = 0) {
		$search_length = mb_strlen($search);
		$replace_length = mb_strlen($replace);
		$offset = 0;
		if( $count == 0 ) {
			while( ($start = mb_strpos($subject, $search, $offset)) !== false ) {
				$subject = mb_substr($subject, 0, $start) . $replace . mb_substr($subject, $start + $search_length);
				$offset = $start + $replace_length;
			}
		} else {
			for( $j = $count; $j > 0; $j-- ) {
				$start = mb_strpos($subject, $search, $offset);
				if( $start !== false ) {
					$subject = mb_substr($subject, 0, $start) . $replace . mb_substr($subject, $start + $search_length);
					$offset = $start + $replace_length;
				} else {
					break;
				}
			}
		}
		return $subject;
	}
}
?>